function CompareFilters()
close all;
% parameters for plotting histogram
nbins = 1000;
dmin = -5;
dmax = 5;
col = ['b','r','m','k'];
% setup figures
set(0,'defaultaxesfontsize',20);
figure(1); hold on; grid;
figure(2); hold on; grid;
% read data and plot them
for i = 1 : 4
    switch i
        case 1
            fname = 'EnKFerror.mat';
        case 2
            fname = 'ETKFerror.mat';
        case 3
            fname = 'PFerror.mat';
        case 4
            fname = 'PFOerror.mat';
        otherwise
            return
    end
    data = load(fname);
    e = data.e;
    rme = data.rme;
    plot_histogram(1,e,nbins,dmin,dmax,col(i));
    figure(2)
    plot(rme,'Linewidth',2,'color',col(i));
end
%
figure(1)
set(gca,'YScale','log');
xlabel('error','Fontsize',20);
ylabel('PDF(error)','Fontsize',20);
legend('EnKF','ETKF','PF','PF Opt');
%
figure(2)
xlabel('iteration','Fontsize',20);
ylabel('Running Mean Squared Error','Fontsize',20);
legend('EnKF','ETKF','PF','PF Opt');
axis([0,length(rme),0,1]);
end
%%
function plot_histogram(fig,data,nbins,dmin,dmax,col)
figure(fig)
Ndata = length(data);
nb1 = nbins + 1;
x = linspace(dmin,dmax,nb1);
h = x(2) - x(1); % bin width
% xc = centers of bins
xc = linspace(dmin + 0.5*h,dmax - 0.5*h,nbins);
hh = zeros(nbins,1); % heights of the bins
for i = 1 : nbins
    ind = find(data >= x(i) & data < x(i + 1));
    hh(i) = length(ind);
end
hh = hh/(h*Ndata); % scale the histogram
plot(xc,hh,'b','Linewidth',2,'color',col);
end
