function KalmanFilter()
close all
set(0,'defaultaxesfontsize',20);
format long
% Code p8.m from Law, Stuart, Zygadakis, page 129
% v_{j + 1} = A*v_j + \xi_j, y_{j + 1} = H*v_{j + 1} + \eta_{j + 1}
%% Setup
J = 1e3; % number of steps
N = 2; % dimension of state
I = eye(N);% identity operator
gamma = 1;% observational noise variance is gamma?2*I
gam2 = gamma^2;
sigma = 1;% dynamics noise variance is sigma?2*I
sig2 = sigma^2;
C0 = eye(2); % prior initial condition variance
m0 =[0;0]; % prior initial condition mean
sd = 10; rng(sd);% choose random number seed
A = [0 1;-1 0]; % dynamics determined by A
% pre-allocate memory for output arrays
m = zeros(N,J); % N = space dimension, J = # of steps
v = m;
y = zeros(J,1);
c = zeros(N,N,J); 
% continue setup
v(:,1) = m0 + sqrtm(C0)*randn(N,1); % initial truth
m(:,1) = 10*randn(N,1);% initial mean/estimate
c(:,:,1) = 100*C0;% initial covariance
H = [1,0]; % observation operator: only the first component is observed
%% Assimilation 
for j = 2 : J 
    j1 = j - 1;
    v(:,j) = A*v(:,j1) + sigma*randn(N,1); % truth
    y(j) = H*v(:,j) + gamma*randn; % observation
    % prediction
    mhat = A*m(:,j1); % estimator predict
    chat = A*c(:,:,j1)*A' + sig2*I; % covariance prediction
    % analysis
    d = y(j) - H*mhat; % innovation
    K = (chat*H')/(H*chat*H' + gam2); % Kalman gain
    m(:,j) = mhat + K*d; % estimator update
    c(:,:,j) = (I - K*H)*chat; % covariance update    
end
%% Visualization
% Plot the second component of the truth that is not observed
figure; hold on; grid;
js = 51; % plot the first 21 steps
t = [0 : js - 1];
ind = [1 : js];
plot(t,v(2,ind),'Linewidth',2);
plot(t,m(2,ind),'m','Linewidth',2);
plot(t,m(2,ind) + reshape(sqrt(c(2,2,ind)),1,js),'r--','Linewidth',2);
plot(t,m(2,ind) - reshape(sqrt(c(2,2,ind)),1,js),'r--','Linewidth',2);
xlabel('iteration, j');
title('Kalman Filter');
legend('v_2','m_2','m_2\pm C_{2,2}^{1/2}')
% Plot the covariance
figure; hold on; grid;
t = [0 : J - 1];
size(reshape(c(1,1,:)+c(2,2,:),J,1))
size(t)
plot(t,reshape(c(1,1,:)+c(2,2,:),J,1)); 
plot(t,cumsum(reshape(c(1,1,:)+c(2,2,:),J,1))./[1:J]','m','Linewidth',2); 
xlabel('iteration, j');
axis([1 1000 0 50]);
title('Kalman Filter Covariance');
legend('Tr(C_k)','(1/j)\sum_{k=1}^jTr(C_k)');
% Plot Kalman filter error
figure; hold on; grid;
plot(t,sum((v-m).^2),'Linewidth',2);
plot(t,cumsum(sum((v-m).^2))./[1:J],'m','Linewidth',2);
xlabel('iteration, j');
axis([1 1000 0 50]);
title('Kalman Filter Error');
legend('|v_j - m_j|^2','(1/j)\sum_{k=1}^j|v_k - m_k|^2')
end